#!/usr/bin/python

import sys, os, re, traceback, string, socket
from optparse import OptionParser, OptionGroup

# add path to modules. It must be first, because we're using more recent standard modules (tempfile) then those in default 2.2.3 installation.
if sys.platform != 'win32':
	sys.path = [os.path.join(os.path.realpath(sys.path[0]), 'modules')] + sys.path
else:
	sys.path = sys.path + [os.path.join(os.path.realpath(sys.path[0]), 'modules')]

import uConfig, uSysDB, uUtil
import ConfigParser

def findDBConfig():
	default_rv = {
		'dbhost' : socket.gethostname(),
		'user' : 'plesk',
		'password' : 'plesk'
	}

	if sys.platform == 'win32':
		return 	default_rv
	else:
		etc_dir = "/usr/local/pem/etc"
		kernel_conf_path = os.path.join(etc_dir, "Kernel.conf")
		odbc_ini_path = os.path.join(etc_dir, "odbc.ini")

		if not os.path.isfile(kernel_conf_path) or not os.path.isfile(odbc_ini_path):
			return default_rv

		kernel_conf = file(kernel_conf_path)
		pp = uUtil.readPropertiesFile(kernel_conf)
		kernel_conf.close()

		odbc = ConfigParser.ConfigParser()
		odbc.read(odbc_ini_path)

		dsnname = pp["dsn"]
		rv = {}
		rv['dbhost'] = odbc.get(dsnname, "servername")
		rv['user'] = pp["dsn_login"]
		rv['password'] = pp['dsn_passwd']
		return rv

parser = OptionParser()

# declare options
common_opts = OptionGroup(parser, "Commands and common options")

common_opts.add_option("--subs-list", dest="sub_list",
	help="The comma-separated list of subscriptions ids. ALL keyword to upgrade all Exchange subscriptions.\nSamples:\n\t./exch2k10sp2_upgrade.py --subs-list 100, 800\n\t./exch2k10sp2_upgrade.py --subs-list ALL")

common_opts.add_option("--subs-file", dest="subs_infile",
	help="The file contains lines with one subscription id per line.\nSamples:\n\t./exch2k10sp2_upgrade.py --subs-file subs.txt")

dbconfig = findDBConfig()
	
common_opts.add_option("--db-user", "-u", dest="db_user",
	help="(E,I) database user to access POA database", metavar="USER", default=dbconfig['user'])
common_opts.add_option("--db-password", "-p", dest="db_password",
	help="(E,I) database password to access POA database", metavar="PASSWORD", default=dbconfig['password'])

common_opts.add_option("--db-host", "-s", dest="db_host",
	help="(E,I) database host to access POA database", metavar="DBHOST", default=dbconfig['dbhost'])

parser.add_option_group(common_opts)

options, arg_list = parser.parse_args()

# check options
if not options.sub_list and not options.subs_infile:
	parser.print_help()
	sys.exit(1)
	
if options.sub_list and options.subs_infile:
    print "--subs-list and --subs-file options are mutually exclusive. Please remove superfluous option."
    sys.exit(4)

# Actually we don't want use this POA Config class but we need fill this
# via other updater modules requirements
config = uConfig.Config()

config.assign(["database", "host"], options.db_host)
config.assign(["database", "user"], options.db_user)
config.assign(["database", "password"], options.db_password)

try:
	uSysDB.init(config)
except Exception, e:
	print "Cannot access to POA DB: "
	print e
	if not options.db_user or not options.db_password:
		print "Specify correct login (--db_user) and password (--db-password) for POA DB, these parameters are mandatory."
	else:
		print "Check POA DB login and password"
	sys.exit(2)


con = uSysDB.connect()
cur = con.cursor()

sql = "UPDATE exch_ds_for_upgrade_al SET force_upgrade = 'y'"

sub_list = options.sub_list
	
if sub_list and sub_list != "ALL":
	sub_list = sub_list.split(',')
elif options.subs_infile:
	print "\nReading file '%s' for subscription list..." % options.subs_infile
	text_file = open(options.subs_infile, "r")
	sub_list = text_file.readlines()
	text_file.close()

# check ids
if sub_list != "ALL":
	for org_id in sub_list:
		org_id = org_id.strip()
		if org_id.isdigit() != 1:
			print "Subscription id '%s' is invalid." % org_id
			sys.exit(3)
	sql += " WHERE ds_id IN (select ds_id FROM exch_domain_services WHERE subscription_id IN (%s))" % ", ".join([sub_id for sub_id in sub_list])

affected_rows = cur.execute(sql)

if sub_list != "ALL" and affected_rows != len(sub_list):
	print "Some of subscriptions are absent or wrong id was passed via --sub-list parameter."
	
if affected_rows is None:
	affected_rows = 0
print "%d subscription(s) scheduled for upgrade." % affected_rows

con.commit()

print "Done."
